#-*- coding: utf-8 -*-

import argparse
import configparser
import os
from shutil import rmtree
from sh import rsync

from l2lss.dcvsapi import Repo
from l2lss.utils import check_is_in_l2lss_repo, get_config_dir, concat_path, get_dcvs_type, get_sync_subfolder

def pull(*sys_args):
    args = parse(*sys_args)
    check_is_in_l2lss_repo()
    # Get config
    config = configparser.ConfigParser()
    l2lss_folder = get_config_dir()
    path_config_file = concat_path(l2lss_folder, 'config')
    config.read(path_config_file)

    # Prepare
    remove_old_sync_folder(l2lss_folder, 10)

    # Sync
    sync_repo(l2lss_folder, config)


def parse(*sys_args):
    """Parse the args from sys.args and returns the result."""
    parser = argparse.ArgumentParser(description='''Pull the central server repo to the local dir.''')
    args = parser.parse_args()

    return args


def remove_old_sync_folder(l2lss_folder, number_to_keep):
    """Make sure that we keep only number_to_keep folders to sync."""
    files = os.listdir(l2lss_folder)
    files.remove('config')
    files.sort()
    if len(files) > number_to_keep:
        number_to_remove = len(files) - number_to_keep
        for folder_to_remove in files[:number_to_remove]:
            rmtree(folder_to_remove)


def sync_repo(l2lss_folder, config):
    """Synchronise the code, files and database."""
    pull_code()
    pull_files(l2lss_folder, config)
    sync_subfolder = concat_path(l2lss_folder, get_sync_subfolder(l2lss_folder))
    sync_files(sync_subfolder, config)
    restore_database(sync_subfolder, config)


def pull_files(l2lss_folder, config):
    """Sync the files from the server."""
    if 'sync path' in config['path']:
        distant_folder = config['path']['sync path']
        if distant_folder[-1] != '/':
            distant_folder += '/'
        rsync(distant_folder, l2lss_folder + '/', r=True, l=True)


def pull_code():
    """Pull the code using the setup of the dcvs."""
    repo_type = get_dcvs_type()
    if repo_type:
        try:
            repo = Repo(type=repo_type)
            repo.pull()
        except Exception:
            print('Repo is unable to pull code. Check that your dcvs is correctly configured.')


def sync_files(sync_subfolder, config):
    """Sync the files of the local site with the files of the sync_subfolder."""
    if 'files dir' in config['files']:
        # Get files_folder
        files_folder = config['files']['files dir']
        if files_folder[-1] != '/':
            files_folder += '/'
        # Get sync files folder
        sync_files_subfolder = os.listdir(sync_subfolder)
        if 'db.sql' in sync_files_subfolder:
            sync_files_subfolder.remove('db.sql')
        source_folder_to_sync = concat_path(sync_subfolder, sync_files_subfolder[0])
        rsync(source_folder_to_sync + '/', files_folder, r=True, l=True, delete=True)


def restore_database(sync_subfolder, config):
    """Restore the database from the sync_subfolder."""
    if 'restore cmd' in config['database']:
        restore_cmd = config['database']['restore cmd'].split(' ')
        restore_cmd_name = restore_cmd[0]

        # We import the command from the sh module in order to use it as a function
        _sh = __import__('sh', globals(), locals(), [restore_cmd_name], 0)
        sh_cmd = getattr(_sh, restore_cmd_name)
        dump_file_name = concat_path(sync_subfolder, '/db.sql')
        if 'encoding' in config['database']:
            encoding = config['database']['encoding']
        else:
            encoding = 'utf-8'
        with open(dump_file_name, 'r', encoding=encoding) as dump_file:
            sh_cmd(*restore_cmd[1:], _in=dump_file)
